# PowerFlux

Since this is a private instance of GitLab and signups are disabled, any questions or comments related to this project can be sent to gitlab@poweron.dk.

## Fetches data and writes to a InfluxDB database

- Meter readings from Eloverblik (Power consumption per hour)
- Spot prices from Energidataservice

#### JSON Output from update.php

Various info will be available, for use in Annotations in Grafana or for whatever you want, see *Example of JSON output* below.

>**Version: v0.56.2**
---

### Manual installation or Docker?

- The [Docker version](#install-using-docker-compose-recommended) includes InfluxDB and Grafana. Grafana is setup with automatic provisioning of datasource and a template dashboard.

- You can of course also use the PHP code on its own if you already have a working PHP 8.2 environment, see *Manual installation* below for instructions.

## [Getting Started](#getting-started)

### [Install using Docker Compose (Recommended)](#install-using-docker-compose-recommended)

#### For the full stack - PHP app, webserver (nginx), database (InfluxDB) and data visualization (Grafana)

##### Build the images yourself with predefined settings

1. Insert your refresh token from Eloverblik as a environment variable in docker-compose_buildFromSource.yml
2. Run Docker Compose on docker-compose_buildFromSource.yml

- Grafana is available on <http://localhost:3000> (username admin, password admin)

- InfluxDB is available on <http://localhost:8086>

- JSON output from PHP app is available on <http://localhost:8181>

#### Already got InfluxDB and Grafana?

Two options:

1. Use docker-compose_prebuilt.yml - This will use prebuilt images(built by the CI/CD pipeline), you will need to set environment variables for the Eloverblik refresh token and connection details for your InfluxDB database, see the file.
  
2. Remove the InfluxDB and Grafana section from docker-compose_buildFromSource.yml and proceed as before.

<details>
<summary>
<b>Manual installation (Alternative install)</b>
</summary>

1. Setup settings.json (inside src/php_app), see [example](#example-of-correct-settingsjson).

2. Run get_ancient.php to get "ancient" data (all years before current) - preferably in CLI to avoid problems with timeouts

3. Run update.php *(for example every 1 hour in a cronjob to ensure continuously data retrieval)*

#### [Requirements](#requirements)

- PHP 8.2
- InfluxDB 2.0+

#### [Example of correct settings.json](#example-of-correct-settingsjson)

```json
{
    "refresh_token": "Your-RefreshToken-from-Eloverblik",</h2>
    "date_from": "",
    "date_to": "",
    "meter_id": "",
    "influxdb_url": "http://192.168.1.2:8086",
    "influxdb_token": "xxxxxxxxxxxxxxxxxxxxxxxxxxx",
    "influxdb_bucket": "YourBucket",
    "influxdb_org": "YourOrganization"
}
```

</details>

<details>
<summary><b>Example of JSON output</b></summary>

### [Example of output (update.php)](#example-of-output-updatephp)

```json
  "info": {
    "nextupdate_readings_in": 16207,
    "nextupdate_readings_dt": "2022-11-15T00:35:19Z",
    "lastupdate_readings_dt": "2022-11-14T16:35:19Z",
    "nextupdate_prices_in": 16262,
    "nextupdate_prices_dt": "2022-11-15T00:36:14Z",
    "lastupdate_prices_dt": "2022-11-14T16:36:14Z",
    "settings_date_from": "2012-07-17",
    "settings_date_to": "2022-11-14",
    "query_date_from": "2022-11-12",
    "query_date_to": "2022-11-14",
    "meters": "521234567890123450"
  }
```

| Key | Description |
| --- | ----------- |
| nextupdate_readings_in | Seconds to next attempt at fetching data from Eloverblik |
| nextupdate_readings_dt | Same as above but in ISO-8601 format (UTC) |
| lastupdate_readings_dt | Last time data was successful retrieved(and stored) from Eloverblik (ISO-8601 format (UTC)) |
| nextupdate_prices_in | Seconds to next attempt at fetching data from Energidataservice |
| nextupdate_prices_dt | Same as above but in ISO-8601 format (UTC) |
| lastupdate_prices_dt | Last time data was successful retrieved(and stored) from Energidataservice (ISO-8601 format (UTC)) |
| settings_date_from | Wil be set after first run to the date of the first available data entry. |
| settings_date_to | Changes automatically to the current date. |
| query_date_from | From date - Time period of last query to Eloverblik. (Changes automatically) |
| query_date_to | To date - Time period of last query to Eloverblik. (Changes automatically) |

</details>

<details>
<summary><b>Notes and Suggestions</b></summary>

- The PHP code and the Docker stack has been tested on Linux(Debian & Photon OS) and Windows 10 (x86-64).
- On Windows you need to escape the paths specified in update.php, with \ or just use / instead, example: ```D:\\aFolder\anotherOne\\test1\\``` and ```D:/aFolder/anotherOne/test1/```should work.

#### Grafana and InfluxDB (Suggestions for manual installation)

If you want to use InfluxQL with InfluxDB 2.0+ you may need to create a DBRP mapping. \
List buckets with: ```influx v1 dbrp list``` \
Create bucket mapping with: ```influx v1 dbrp create --bucket-id yourBucketID --db yourBucket --rp autogen``` \
<https://docs.influxdata.com/influxdb/v2.5/query-data/influxql/dbrp/#create-dbrp-mappings> \
This only applies to InfluxQL, for Flux queries you don't need to do anything.

 Suggested plugins for using JSON output as a datasource:

- Infinity - <https://grafana.com/grafana/plugins/yesoreyeram-infinity-datasource/>
- JSON API - <https://grafana.com/grafana/plugins/marcusolsson-json-datasource/>

##### Delay on data retrieval

 update.php will attempt to fetch new data from Eloverblik every 6 hours (*updateinterval* in Configuration class), for the latest entry in the InfluxDB database to the current date. \
 Please note that Eloverblik will always provide data with 2~ days delay. \
You can call this code every few seconds if you like to, it will only do calls to Eloverblik after 6 hours has passed since last successful request. \
Docker version runs update.php every minute (a bit overkill yes).

##### Code quality

The code is still a work in progress and there is definitely room for improvement but its stable and has been working without any bugs or problems the last 2 years*.
(*Which is also the reason for the dramatic decline in development on this project)

</details>
