<?php

namespace PowerFlux;

use DateInterval;
use DateTimeImmutable;

class Common
{

    const DATEHUMAN = "Y-m-d H:i:s \(\U\T\C\)"; //Eaiser for the eyes than 8601 or c

    /**
     * Checks if $input has somewhat valid JSON.
     */
    public static function hasJSON($input)
    {
        if (!empty($input)) {
            $j = json_decode($input, true);
            if (json_last_error() === JSON_ERROR_NONE) {
                return $j;
            }
        }
        return false;
    }


    //Todo: clean up
    /**
     * is DateTime(Immutable) after DateTime(Immutable)?
     */
    public static function isDateAfter($compareto, $newdate): bool
    {

        if (($compareto instanceof \DateTimeImmutable) | ($compareto instanceof \DateTime)) {
            $compareto = $compareto->format("c");
        }
        if (($newdate instanceof \DateTimeImmutable) | ($newdate instanceof \DateTime)) {
            $newdate = $newdate->format("c");
        }
        $compareto = Common::getDateTime($compareto);
        $newdate = Common::getDateTime($newdate);
        if ($compareto < $newdate) {
            return true;
        }
        return false;
    }


    /** Expects start and end dates as string. //Todo: remove
     * Returns array of 1-year periods (YYYY-01-01T00:00:00+00:00 -> YYYY-12-31T23:59:59+00:00)
     */
    public static function getAncientPeriods($start, $end, $format = "c"): ?array
    {
        $start = Common::getDateTime($start) ?? exit(ErrorOutput::handleError(ERRORCODES::UNKNOWN));
        $end = Common::getDateTime($end) ?? exit(ErrorOutput::handleError(ERRORCODES::UNKNOWN));

        $period = new \DatePeriod($start, new DateInterval("P1Y"), $end);
        $r = array();

        foreach ($period as $year) {
            $yearstart = $year;
            $yearstart = $year->modify("first day of January");
            $yearend = $year->modify("last day of December");
            $yearend = $yearend->modify("23:59:59");

            if ($yearend->format("Y") != Common::getTimeDate(format: "Y")) {
                if (!Configuration::doIgnore(type: "prices", year: $yearend->format("Y"))) { //If year is not in ignore list
                    array_push($r, array("start"=>$yearstart->format($format), "stop"=> $yearend->format($format) ) );
                }
            }
        }
        return $r ?? null;
    }



    /** Expects start and end dates as string.
     * Returns array of 1-year periods as string(default: YYYY-01-01T00:00:00+00:00 -> YYYY-12-31T23:59:59+00:00)
     */
    public static function getPeriods($start, $end, $format = "c"): ?array
    {
        $start = Common::getDateTime($start) ?? exit(ErrorOutput::handleError(ERRORCODES::UNKNOWN));
        $end = Common::getDateTime($end) ?? exit(ErrorOutput::handleError(ERRORCODES::UNKNOWN));

        $period = new \DatePeriod($start, new DateInterval("P1Y"), $end);
        $r = array();

        foreach ($period as $year) {

            if ($year->format("Y") != Common::getTimeDate(format: "Y")) { //If period is not this year, modify start and end
                $yearstart = $year;
                $yearstart = $year->modify("first day of January");
                $yearend = $year->modify("last day of December");
                $yearend = $yearend->modify("23:59:59");
            } else {
                $yearstart = $year->modify("first day of January"); //Dosen't get used
                $yearend = Common::getDateTime("now");
                $yearend = $yearend->modify("+1 day"); //Tomorrow
            }

            array_push($r, array("start"=>$yearstart->format($format), "stop"=> $yearend->format($format) ) );
        }

        //tmp fix - Todo: Rework above
if ( Common::getTimeDate(str: end($r)["start"], format: "Y") !== Common::getTimeDate(format: "Y") ) {
$thisyear = Common::getDateTime("now");
$thisyear = $thisyear->modify("first day of January");
$thisyear = $thisyear->modify("00:00:00");
$end = $end->modify("+1 day"); //Tomorrow
array_push($r, array("start"=>$thisyear->format($format), "stop"=> $end->format($format) ) );
}

        return $r ?? null;
    }



     /**
     * Returns additional data for use in output.
     */
    public static function getInfoData(): ?array
    {
        $json = array();

        if ($t = Common::getTimeDifference(Configuration::getAppEntry("ts_api_readings_resp"), true, Configuration::getUpdateIntervalforReadings())) {
            $json["nextupdate_readings_in"] = $t;
        };

        if ($t = \DateTime::createFromFormat("U", Common::getTime()+Common::getTimeDifference(Configuration::getAppEntry("ts_api_readings_resp"), true, Configuration::getUpdateIntervalforReadings()) )) {
            $json["nextupdate_readings_dt"] = $t->format("Y-m-d\TH:i:s\Z");
        };

        if ($t = \DateTime::createFromFormat("U", Configuration::getAppEntry("ts_api_readings_resp") )) {
            $json["lastupdate_readings_dt"] = $t->format("Y-m-d\TH:i:s\Z");
        };

        if ($t = Common::getTimeDifference(Configuration::getAppEntry("ts_api_prices_resp"), true, Configuration::getUpdateIntervalforPrices())) {
            $json["nextupdate_prices_in"] = $t;
        };

        if ($t = \DateTime::createFromFormat("U", Common::getTime()+Common::getTimeDifference(Configuration::getAppEntry("ts_api_prices_resp"), true, Configuration::getUpdateIntervalforPrices()) )) {
            $json["nextupdate_prices_dt"] = $t->format("Y-m-d\TH:i:s\Z");
        };

        if ($t = \DateTime::createFromFormat("U", Configuration::getAppEntry("ts_api_prices_resp") )) {
            $json["lastupdate_prices_dt"] = $t->format("Y-m-d\TH:i:s\Z");
        };

        if ($t = Configuration::getSettingsEntry("date_from")) {
            $json["settings_date_from"] = $t;
        };

        if ($t = Configuration::getSettingsEntry("date_to")) {
            $json["settings_date_to"] = $t;
        };

        if ($t = Configuration::getAppEntry("query_date_from")) {
            $json["query_date_from"] = $t;
        };

        if ($t = Configuration::getAppEntry("query_date_to")) {
            $json["query_date_to"] = $t;
        };

        if ($t = Configuration::getSettingsEntry("meter_id")) {
            $json["meters"] = $t;
        };

        return $json;    
    }


    /** Return DateTimeImmutable with thisYear-01-01 00:00:00
     * 
    */
    public static function getStartDateTime(): ?DateTimeImmutable
    {
        $dt = Common::getDateTime("now")->modify("first day of January");
        $dt = $dt->setTime(0, 0, 0);
        return $dt ?? null;
    }

    /**
     * Returns DateTimeImmutable from Unix timestamp.
     */
    public static function getDateTimeFromTS(?int $input): ?DateTimeImmutable
    {
        if ($input) {
            $r  = new \DateTimeImmutable("@" . $input, new \DateTimeZone("UTC"));
        }
        return $r ?? null;
    }

    /**
     * Returns DateTimeImmutable from string $input. Always UTC.
     */
    public static function getDateTime($input): ?DateTimeImmutable
    {
        if ($input) {
            $r = new \DateTimeImmutable($input, new \DateTimeZone("UTC"));
        }
        return $r ?? null;
    }


    /** Get time difference between two DateTimeImmutable objects.
     * In sec, min or hour (default: min)
     * Returns int or null
    */
    public static function getTimeDiff(
        ?\DateTimeImmutable $time1 = new \DateTimeImmutable("now", new \DateTimeZone("UTC")),
        ?\DateTimeImmutable $time2 = new \DateTimeImmutable("now", new \DateTimeZone("UTC")),
        $as = "min"): ?int
        {

            if ($time1 instanceof \DateTimeImmutable && $time2 instanceof \DateTimeImmutable) {
                $diff = ($time1->getTimestamp())-($time2->getTimestamp());

                switch ($as) {

                    case "sec":
                        return $diff;
                        break;

                    case "min":
                        return $diff/60;
                        break;
                
                    case "hour":
                        return $diff/60/60;
                        break;

                }
            }
        return null;
    }

    /**
     * Get time difference in seconds between two Unix timestamps.  //Todo, remove
     */
    public static function getTimeDifference($compareto, bool $subtract = false, $inputb = 0)
    {
        $dtnow = self::getTime();
    
        if (!empty($compareto) | !empty($dtnow)) {
            if (!$subtract) {
                $timediff = abs( ($compareto)-($dtnow) );
            } else { //Do subtraction
                $timediff =  $inputb-(($dtnow)-($compareto))  ;
            }
            return (int)$timediff;
        } else {
            return false;
        }
    }

    /**
     * Returns current time in Unix timestamp format as int (UTC)
     */
    public static function getTime(): ?int
    {
        $dtnow = new \DateTime("now", new \DateTimeZone("UTC"));
        $dtnow = $dtnow->getTimestamp();
        return $dtnow;
    }

    /**
     * Returns date/time as string (default: now() YYYY-MM-DD format (UTC))
     */
    public static function getTimeDate(?string $str = "now", string $format = "Y-m-d", string $tz = "UTC"): string|bool
    {
        try {
            $dt = new \DateTimeImmutable($str, new \DateTimeZone($tz));
            $dt = $dt->format($format);

            if ($dt) {
                return $dt;
            }
        }
        catch (\Exception $e) {
          ErrorOutput::handleError(ERRORCODES::UNKNOWN, $e->getMessage());
        }
        return false;
    }


}
