<?php

namespace PowerFlux;

use PowerFlux\Files\File;
use PowerFlux\Files\FileApp;
use PowerFlux\Files\FileSettings;

class Configuration
{

    private static $apiurl = "https://api.eloverblik.dk/customerapi/";
    private static $apiurl_prices = "https://api.energidataservice.dk/";
    private static $updateinterval_readings = 360; //mins - Interval between attempts at fetching live data from Eloverblik (6 hours)
    private static $updateinterval_prices = 60; //mins - Interval between attempts at fetching live data from Energidataservice (1 hour)
    private static $path_settings = "";
    private static $path_app = "";
    private static $path_data_response = "/private/"; //__DIR__ . "/../../";
    private static $path_log = "";
    private static $file_settings = "settings.json";
    private static $file_app = "app.json";
    private static $file_data_response = "data_response.json";
    private static $file_log = "log.txt";
    private static $toggleoutput = false;
    private static $toggledebug = false;
    private static $togglelog = false;
    //private static $api_accesstoken = null;
    private static $_instance;

    public const REQUEST_DELAYPERIOD = 90;
    public const defaultpath = "/private/";


    public function __construct()
    {
        self::getDebug();
    }

    public static function getApiUrl(): string
    {
        return self::$apiurl;
    }

    public static function getApiUrl_Prices(): string
    {
        return self::$apiurl_prices;
    }


    public static function getUpdateIntervalforReadings(): ?int
    {
        if (self::$updateinterval_readings) {
            $secs = (self::$updateinterval_readings) * 60;
        }
        return $secs ?? null;
    }

    public static function getUpdateIntervalforPrices(): ?int
    {
        if (self::$updateinterval_prices) {
            $secs = (self::$updateinterval_prices) * 60;
        }
        return $secs ?? null;
    }

    public static function getRefreshToken() //Todo: Check if valid token
    {
        return self::getSettingsEntry("refresh_token");
    }

    public static function getSettingsEntry($key)
    {
        if ($file = FileSettings::getFile()) {
            $json = json_decode($file, true);

            if (empty($key) || !is_array($json)) {
                return null;
            }
            if (array_key_exists($key, $json)) {

                switch ($key) {

                    case "refresh_token":
                        if (strlen($json[$key]) > 25) {
                            return $json[$key];
                        }
                        break;

                    case "date_from": //Todo: Check for valid date format (YYYY-MM-DD)
                        if (strlen($json[$key]) === 10) {
                            return $json[$key];
                        }
                        break;

                    case "date_to": //Todo: Check for valid date format (YYYY-MM-DD)
                        if (strlen($json[$key]) === 10) {
                            return $json[$key];
                        }
                        break;

                    case "meter_id": //Todo: Check if value actually is valid numbers
                        if (strlen($json[$key]) >= 10) {
                            return $json[$key];
                        }
                        break;

                    case "influxdb_url":
                        if ($r = Influxdb::isValidURL($json[$key])) {
                            return $r;
                        }
                        break;


                    case "influxdb_token":
                        if (strlen($json[$key]) >= 2) {
                            return $json[$key];
                        }
                        break;


                    case "influxdb_bucket":
                        if (strlen($json[$key]) >= 1) {
                            return $json[$key];
                        }
                        break;

                    case "influxdb_org":
                        if (strlen($json[$key]) >= 1) {
                            return $json[$key];
                        }
                        break;


                    default:
                        return null;
                }
            } else {
                ErrorOutput::DEBUG("Missing key in settings.json: " . $key ?? "null");
            }
            return null;
        }
        return null;
    }

    public static function setSettingsEntry($key, $input): ?bool
    {
        if ($file = FileSettings::getFile()) {
            $array = json_decode($file, true);
            if (!empty($key) && is_array($array)) {
                if (array_key_exists($key, $array)) {
                    $array[$key] = strval($input);  //Todo: influxdb_url escapes?
                    $json = json_encode($array, JSON_PRETTY_PRINT);
                    if (File::saveFile(type: "settings", input: $json)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }


    public static function setAppEntry(string $key, $input): bool
    { //Todo: Any $input should be checked before this get called
        if ($file = FileApp::getFile()) {
            $array = json_decode($file, true);

            if (array_key_exists($key, $array)) {
                $array[$key] = $input;
                $json = json_encode($array, JSON_PRETTY_PRINT);
                if (FileApp::saveFile(type: "app", input: $json)) {
                    return true;
                }
            }
        }
        return false;
    }

    public static function doIgnore(string $type, string $year): ?bool
    {
        $appentry = self::getIgnoreType($type);
        $array = Configuration::getAppEntry($appentry);

        if ($array) {
            if ((in_array($year, $array)) === true) {
                ErrorOutput::DEBUG("Year " . $year . " was found in ignore list for " . $type . ".");
                return true;
            } else {
                ErrorOutput::DEBUG("Year " . $year . " was NOT found in ignore list for " . $type . ".");
                return false;
            }
        }
        
        return null;
    }

    public static function getIgnoreType(string $type): string|false
    {
        switch ($type) {

            case "readings":
                return "ignore_readings";
                break;

            case "prices":
                return "ignore_prices";
                break;

            default:
                return false;
        }
    }

    public static function addToIgnore(string $type, string $year)
    {
        $appentry = self::getIgnoreType($type);
        $array = Configuration::getAppEntry($appentry);

        if (in_array($year, $array) === false) {
            array_push($array, $year);
            Configuration::setAppEntry($appentry, $array);
            ErrorOutput::INFO("Adding " . $year . " to ignore list for " . $type . ".");
        }
    }


    public static function getAppEntry(string $key): string|array|null //Todo: rework
    {
        if ($file = FileApp::getFile()) {
            $json = json_decode($file, true);

            if (array_key_exists($key, $json)) {

                switch ($key) {

                    case "access_token":
                        if (strlen($json[$key]) > 25) {
                            return $json[$key];
                        }
                        break;

                    case "query_date_from":
                        if (strlen($json[$key]) === 10) {
                            return $json[$key];
                        }
                        break;

                    case "query_date_to":
                        if (strlen($json[$key]) === 10) {
                            return $json[$key];
                        }
                        break;

                    case "ts_api_readings_resp":
                        if (strlen($json[$key]) >= 8) {
                            return $json[$key];
                        }
                        break;

                    case "ts_api_readings_lastfail":
                        if (strlen($json[$key]) >= 8) {
                            return $json[$key];
                        }
                        break;

                    case "ts_api_prices_resp":
                        if (strlen($json[$key]) >= 8) {
                            return $json[$key];
                        }
                        break;

                    case "ts_api_prices_lastfail":
                        if (strlen($json[$key]) >= 8) {
                            return $json[$key];
                        }
                        break;

                    case "ignore_readings":
                        return $json[$key];
                        break;

                    case "ignore_prices":
                        return $json[$key];
                        break;

                    default:
                        return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
        return null;
    }

    public static function setAccessToken(string $str)
    {
        if ($str) { //Todo: Consider using JWT decoder(in SET->jwt check->r true) to validate token
            //self::$api_accesstoken = $str;
            self::setAppEntry("access_token", $str);
        }
    }

    /** Returns access token from file or fetchToken() if empty
     * 
     */
    public static function getAccessToken(): string|bool
    {
        if (!self::getAppEntry("access_token")) { //No access token
            $resp = Parser::fetchToken(); //Request new access token

            if ($resp) {
                self::setAccessToken($resp);
                return $resp;
            }
        } else { //Access token is present in app file
            $t = self::getAppEntry("access_token");
            if ($t) {
                return $t;
            }
        }
        return false;
    }

    public static function getMeters(): ?string
    {
        if ($meter = Configuration::getSettingsEntry("meter_id")) {
            $r = $meter;
        } else {
            $r = Parser::fetchMeters();
            Configuration::setSettingsEntry("meter_id", $r);
        }
        return $r ?? null;
    }


    #region Toggles
    public static function setToggleLog(bool $bool)
    {
        self::$togglelog = $bool;
    }

    public static function getToggleLog(): bool
    {
        return self::$togglelog;
    }

    public static function setToggleOutput(bool $bool)
    {
        self::$toggleoutput = $bool;
    }

    public static function getToggleOutput(): bool
    {
        if ($a = self::$toggleoutput) {
            return $a;
        }
        return false;
    }

    public static function setDebug(bool $bool)
    {
        self::$toggledebug = $bool;
        if ($bool) {
            self::getDebug();
        }
    }

    public static function getDebug(): bool
    {
        if (self::$toggledebug) {
            ini_set('display_errors', '1');
            ini_set('display_startup_errors', '1');
            ini_set('assert.exception', '1');
            ini_set('max_execution_time', '240'); //5
            ini_set('default_socket_timeout', '240');
            error_reporting(E_ALL ^ E_DEPRECATED); //^ E_DEPRECATED

            self::setSettingsPath(__DIR__ . "/../../test/");
            self::setLogPath(__DIR__ . "/../../test/");
            self::setAppPath(__DIR__ . "/../../test/");
            self::$path_data_response = __DIR__ . "/../../test/";
        }
        return self::$toggledebug;
    }
    #endregion


    #region Files
    public static function setSettingsPath($path)
    {
        self::$path_settings = $path;
    }

    public static function setAppPath($path)
    {
        self::$path_app = $path;
    }

    public static function setLogPath($path)
    {
        self::$path_log = $path;
    }

    public static function getLogFile(): ?string
    { //Todo:
        return null;
    }

    public static function getFilePath(string $type): string|false
    {

        switch ($type) {
            case "app":
                return self::$path_app . self::$file_app;
                break;

            case "settings":
                return self::$path_settings . self::$file_settings;
                break;

            case "dataresp":
                return self::$path_data_response . self::$file_data_response;
                break;

            case "log":
                return self::$path_log . self::$file_log;
                break;
        }

        return false;
    }
    #endregion


    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new Configuration();
        }
        return self::$_instance;
    }
}
