<?php

namespace PowerFlux;

use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;

class Prices
{

    private static $_instance;

    public function __construct()
    {
    }

    /**
     * Checks if enough time has passed since last unsuccessful request (90s)
     */
    private static function doAttempt(): bool
    {
        
        if ($ts = Configuration::getAppEntry("ts_api_prices_lastfail")) {
            $timediff = Common::getTimeDifference($ts);

            if ($timediff) {
                if ($timediff >= Configuration::REQUEST_DELAYPERIOD) {
                    return true;
                } else {
                    return false;
                }
            }
        }
        return true;
    }

    private static function doPriceFetch(): bool
    {
        $datats = Configuration::getAppEntry("ts_api_prices_resp") ?? 1;
        $timediff = round((Configuration::getUpdateIntervalforPrices()-(abs(Common::getTimeDiff( Common::getDateTimeFromTS($datats), as: "sec"))))/60,0);
        $timesince = abs(Common::getTimeDiff(Common::getDateTimeFromTS($datats), as: "sec"));

        ErrorOutput::DEBUG("DELAY PERIOD, NEXT FETCH IN: " . $timediff. " minutes");
        ErrorOutput::DEBUG("DELAY PERIOD, WAIT? -> LAST FETCH WAS " . round($timesince/60,0) . " minutes ago");
        ErrorOutput::DEBUG("GRACE PERIOD, DOATTEMPT: " . Prices::doAttempt() );

        if (($timesince > Configuration::getUpdateIntervalforPrices()) && (Prices::doAttempt())) { //Data is old & last attempt was more than X seconds ago, fetch new
            ErrorOutput::INFO("ACTION : LAST FETCH WAS: " . round($timesince/60,0) . " minutes ago");
            return true;
        } else {
            ErrorOutput::DEBUG("STOP: NOT RUNNING - WAITING " . $timediff. " minutes");
        }
        return false;
    }

    public static function doTimeSeriesPrices(string $input) //Todo: rework
    {
        $dataout = array();

        if (!empty($input)) {
            $array = json_decode($input, true);
            if ($array["dataset"] == "Elspotprices") { //If dataset is Elspotprices
                if (array_key_exists("records", $array)) { //If "records" key exists


                    foreach ($array["records"] as $hour) { //Loop trough records
                        $dt = Common::getDateTime($hour["HourUTC"]);
                        array_push($dataout, array("date" => $dt->format("c"), "priceDKK" => $hour["SpotPriceDKK"], "priceArea" => $hour["PriceArea"] ));
                    }

                    $arrayout["data"] = $dataout;
                }
            }
        }

        return $arrayout ?? null;
    }


    public static function fetchElspotPrices($periodstart = "", $periodend = ""): ?string
    {
        $apiurl = Configuration::getApiUrl_Prices();

        try {
            $filters = array("PriceArea" => "DK1,DK2");
            $filters = json_encode($filters);

            if (empty($periodstart) | empty($periodend) ) {
                exit("nope");
            }

            ErrorOutput::DEBUG("FETCH PRICES, for period " . $periodstart . " -> " . $periodend);

            $client = new \GuzzleHttp\Client(['base_uri' => $apiurl]);
            $response = $client->request("GET", "dataset/Elspotprices", [
                "headers" => [
                    "user-agent" => "guzzlehttp/1.0",
                    "accept"     => "application/json"
                ],
                "query" => [
                    "start" => $periodstart,
                    "end" => $periodend,
                    "filter" => $filters,
                    "limit" => "0",
                    "columns" => "HourUTC, PriceArea, SpotPriceDKK",
                    "timezone" => "UTC"
                ]
            ]);

            if ($response->getStatusCode() == "200") {
                $resp = (string)$response->getBody();
                Configuration::setAppEntry("ts_api_prices_resp", Common::getTime());
                return $resp;
            }
        } catch (ClientException $e) { //4xx responses
            $exresp = $e->getResponse();
            Configuration::setAppEntry("ts_api_prices_lastfail", Common::getTime());
            ErrorOutput::handleError(ERRORCODES::tryFrom($exresp->getStatusCode()) ?? ERRORCODES::UNKNOWN);
        } catch (ServerException $e) { //5xx responses
            $exresp = $e->getResponse();
            Configuration::setAppEntry("ts_api_prices_lastfail", Common::getTime());
            ErrorOutput::handleError(ERRORCODES::tryFrom($exresp->getStatusCode()) ?? ERRORCODES::UNKNOWN);
        }

        return false;
    }


    public static function doPrices()
    {
        $startperiod = Configuration::getSettingsEntry("date_from");
        $endperiod = Configuration::getSettingsEntry("date_to");

        $periods = Common::getPeriods($startperiod, $endperiod, format: "Y-m-d\TH:i"); //Get 1 year periods for the specified period

        if (Prices::doPriceFetch()) { //Time to fetch?


            ErrorOutput::DEBUG("ACTION -> Loop periods");
            foreach ($periods as $year) {
                $yearend = Common::getDateTime($year["stop"])->format("Y");

                if (!Configuration::doIgnore(type: "prices", year: $yearend) ) { //If year not in ignore list
 
                    if ($yearend == Common::getDateTime("now")->format("Y") ) { //If period is in this year
                        if ($lastentry = (Influxdb::getLatestReading(type: "prices")?->format("Y-m-d\TH:i") ?? Common::getStartDateTime()->format("Y-m-d\TH:i") ) ) { //If null get start of this year
                            $r =  Prices::doTimeSeriesPrices(Prices::fetchElspotPrices(periodstart: $lastentry, periodend: $year["stop"])); //Get data for last entry and tomorrows date(from getPeriods)
                            Influxdb::doWriteForPrices($r);
                        }
                    } else {
                        $r =  Prices::doTimeSeriesPrices(Prices::fetchElspotPrices(periodstart: $year["start"], periodend: $year["stop"]));
                        if (Influxdb::doWriteForPrices($r)) { //If write succeeds, add the year to the ignore list
                            Configuration::addToIgnore(type: "prices", year: $yearend); //Todo: this or find a good way to check the database
                        }
                    }
 
                 
 
                }
            }


        }
    }



    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new Prices();
        }
        return self::$_instance;
    }
}
