<?php
namespace PowerFlux;

require_once(__DIR__ . '/../vendor/autoload.php');
header('content-type: application/json');

class UpdateAncientApp
{

    public function __construct()
    {

        //User Options
        Configuration::setToggleOutput(true); //Outputs various info in JSON format : true/false
        Configuration::setToggleLog(true); //Write to log file : true/false
        $loglevels = array("DEBUG", "INFO", "NOTICE", "WARNING", "ERROR"); // Possible log levels - default INFO
        ErrorOutput::setLogLevel("INFO");

        //If no custom path is set the files will be placed in the /private folder
        Configuration::setSettingsPath(Configuration::defaultpath);
        Configuration::setLogPath(Configuration::defaultpath);
        Configuration::setAppPath(Configuration::defaultpath);
        //For modifying the path for settings.json, log.txt and app.json, replace the above with, for example:
        // Configuration::setSettingsPath("/var/www/clients/client0/web20/private/elo/");

        //End of User Options

        Configuration::setDebug(false); //PHP debugging & read/writes to php_app\test folder








        //App code
        $env_debug = getenv("DEBUG");
        if ($env_debug == "true" || $env_debug == "false") {
            $_bool = filter_var($env_debug, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE) ?? false;
            Configuration::setDebug($_bool);
        }
        $env_loglevel = getenv("LOGLEVEL");
        if (in_array($env_loglevel, $loglevels)) {
            ErrorOutput::setLogLevel($env_loglevel);
        }


        if (Configuration::getSettingsEntry("refresh_token") === false) {
            Configuration::setSettingsEntry("refresh_token", getenv('REFRESH_TOKEN'));
        }

        if (Configuration::getSettingsEntry("influxdb_url") === false) {
            Configuration::setSettingsEntry("influxdb_url", getenv('INFLUXDB_URL'));
        }

        if (Configuration::getSettingsEntry("influxdb_token") === false) {
            Configuration::setSettingsEntry("influxdb_token", getenv('INFLUXDB_TOKEN'));
        }

        if (Configuration::getSettingsEntry("influxdb_bucket") === false) {
            Configuration::setSettingsEntry("influxdb_bucket", getenv('INFLUXDB_BUCKET'));
        }
        if (Configuration::getSettingsEntry("influxdb_org") === false) {
            Configuration::setSettingsEntry("influxdb_org", getenv('INFLUXDB_ORG'));
        }



        if (Configuration::getSettingsEntry("date_from") === false) {
            Configuration::setSettingsEntry("date_from", "2000-01-01");
        }
        Configuration::setSettingsEntry("date_to", Common::getTimeDate()); // Set date to current date


        $meter = Configuration::getMeters();
        $startperiod = Configuration::getSettingsEntry("date_from");
        $endperiod = Configuration::getSettingsEntry("date_to");

        $periods = Common::getAncientPeriods($startperiod, $endperiod);
        $r = Influxdb::doAncientPeriods(periods: $periods, meter: $meter);

        $j = array();
        $j["periods"] = $periods;
        $j["info"] = Common::getInfoData();
        echo json_encode($j, JSON_PRETTY_PRINT);
    }
}


new UpdateAncientApp();
